// Fill out your copyright notice in the Description page of Project Settings.


#include "WaypointActor.h"

#include "EndlessVendetta/EndlessVendettaCharacter.h"

// Sets default values
AWaypointActor::AWaypointActor()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
}

// Called when the game starts or when spawned
void AWaypointActor::BeginPlay()
{
	SetActorTickEnabled(false);
	Super::BeginPlay();
	
	SetActorTickInterval(0.05);
	AActor* PlayerActor = Cast<AActor>(GetWorld()->GetFirstPlayerController()->GetPawn());
	PlayersCam = Cast<UCameraComponent>(PlayerActor->GetComponentByClass(UCameraComponent::StaticClass()));
	ScalingMagnitude = (ScaleAtMaxDist - ScaleAtMinDist) / (MaxDist - MinDist);
	ScalingY_Intercept = ScaleAtMaxDist - (MaxDist * ScalingMagnitude);
	if (PlayersCam) SetActorTickEnabled(true);
}

// Called every frame
void AWaypointActor::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
	
	if (!PlayersCam) SetActorTickEnabled(false);

	FVector WaypointLoc = GetActorLocation();
	FVector PlayerCamLoc = PlayersCam->GetComponentLocation();
	
	FRotator NewRot = UKismetMathLibrary::FindLookAtRotation(WaypointLoc, PlayerCamLoc);
	SetActorRotation(NewRot);

	float DistanceToPlayer = FVector::Distance(WaypointLoc, PlayerCamLoc);
	// S for Scale
	float S = (DistanceToPlayer * ScalingMagnitude) + ScalingY_Intercept;
	S = FMath::Clamp(S, ScaleAtMinDist, ScaleAtMaxDist);
	SetActorScale3D(FVector(1.f, S, S));
}

